# Copyright 2021 Benedict Suska <benedict.suska@teckids.org>
# Copyright 2021 Dominik George <dominik.george@teckids.org>
# Copyright 2022 Martin Winter <martin.winter@teckids.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import time
from collections import OrderedDict
from mytrix import Matrix
from random import getrandbits
from machine import Timer

# Initialise hardware and framebuf before importing modules.
from color_setup import ssd  # Create a display instance
from gui.core.nanogui import refresh
from gui.core.colors import *

def set_colour(matches, event):
    """Set the RGB LED colour depending on the colour name in a message."""
    colour = matches.group(1).lower().strip()
    sender = event["sender"]

    # Mapping colour names to RGB values
    colour_map = {
        "schwarz": BLACK,
        "blau": BLUE,
        "rot": RED,
        "gruen": GREEN,
        "tuerkis": CYAN,
        "grau": GREY,
        "gelb": YELLOW,
        "rosa": MAGENTA,
        "weiss": WHITE
    }

    if is_allowed(sender):
        if not colour in colour_map:
            # The colour name is not in the mapping
            matrix.send_room_message(
                ROOM_ID, "Diese Farbe kenne ich leider nicht, %s :(." % (sender,)
            )
            return

        ssd.fill(colour_map[colour])
        refresh(ssd)

        matrix.send_room_message(ROOM_ID, "Ich leuchte jetzt %s, %s!" % (colour, sender))


def is_allowed(sender):
    """Check whether a sender is allowed to control the cheese."""
    with open("users.txt", "rt") as f:
        users = f.readlines()

    if "%s\n" % (sender,) in users:
        return True
    else:
        matrix.send_room_message(ROOM_ID, "Du hast mir gar nichts zu sagen, %s!" % (sender,))
        return False


def allow_user(matches, event):
    """Add a user to the allow list."""
    user = matches.group(1)
    sender = event["sender"]

    if is_allowed(sender):
        with open("users.txt", "at") as f:
            f.write(user + "\n")
        matrix.send_room_message(ROOM_ID, "%s darf mich jetzt leuchten lassen!" % (user,))


def not_stupid(matches, event):
    """Say that the cheese is not stupid."""
    sender = event["sender"]
    matrix.send_room_message(ROOM_ID, "Ich bin nicht bloed, %s!" % (sender,))
#    blink((255, 0, 0), 5, 0.3)


def party(matches, event):
    """Blink wildly."""
    sender = event["sender"]
    if is_allowed(sender):
        matrix.send_room_message(
            ROOM_ID, "Party! Party! Party!"
        )
        for _ in range(10):
            r = getrandbits(8)
            g = getrandbits(8)
            b = getrandbits(8)
            blink((r, g, b), 2, 0.3)


def off(matches, event):
    """Turn LEDs off."""
    sender = event["sender"]
    if is_allowed(sender):
        matrix.send_room_message(ROOM_ID, "Nerv ich? Na gut...")
        ssd.fill(BLACK)
        refresh(ssd)

def fallback(matches, event):
    """Fallback fo unknown messages."""
    sender = event["sender"]
    matrix.send_room_message(ROOM_ID, "Was willst du, %s?" % (sender,))

def blink(rgb, times, pause):
    """Blink in rgb color, times times, with pause ms delay."""
    r, g, b = rgb
    for _ in range(times):
        ssd.fill(ssd.rgb(r,g,b))
        refresh(ssd)
        time.sleep(pause)
        ssd.fill(BLACK)
        refresh(ssd)
        time.sleep(pause)

# Connect to Matrix server
matrix = Matrix()
ROOM_ID = "!RpVTffcloKQJGuJdvg:matrix.org"
NICK = "Lampe"

# On which texts to react and which LEDs to turn on
cases = OrderedDict()
cases["^%s, leuchte (.*)" % (NICK,)] = set_colour
cases["^%s, hoer auf (.*)" % (NICK,)] = allow_user
cases["^%s, aus" % (NICK,)] = off
cases["bloede lampe"] = not_stupid
cases["party"] = party
cases["^%s.*" % (NICK,)] = fallback

def do_react():
    matrix.react_room_messages(ROOM_ID, cases, regex=True)
    print("Hello you!")

# Set my nickname and avatar
matrix.set_displayname(NICK)

# Join the room
matrix.join_room(ROOM_ID)

# Send message that I am there
matrix.send_room_message(ROOM_ID, "Hey! Alles klar, ich bin bereit zum Leuchten!")
matrix.send_room_message(ROOM_ID, "Lass mir etwas Zeit, auf deine Nachrichten zu reagieren!")

    
tim1 = Timer(2)
tim1.init(period=5*1000, mode=Timer.PERIODIC, callback=lambda t:do_react())